"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var _DB = require("./db/db").DB;

var Client = require("./websocket/websocket-client");

var Realtime = require("./realtime/realtime");

var _FileStore = require("./filestore/filestore");

var fetchAsync = require("./utils").fetchAsync;
/**
 * The DB Client Interface.
 * @external DB
 * @see {@link https://github.com/spaceuptech/space-api-js/wiki/DB}
 */

/**
* The Realtime Client Interface.
* @external Realtime
* @see {@link https://github.com/spaceuptech/space-api-js/wiki/Realtime}
*/

/**
* The FileStore Client Interface.
* @external FileStore
* @see {@link https://github.com/spaceuptech/space-api-js/wiki/FileStore}
*/

/**
 * Class representing the client api.
 * @example
 * import { API } from 'space-api';
 *
 * const api = new API('my-project', 'http://localhost:4122');
 */


var Api =
/*#__PURE__*/
function () {
  /**
   * Create an instance of the Client API.
   * @param {string} projectId - The Project Id.
   * @param {string} url - Base url of space-cloud server.
   */
  function Api(projectId) {
    var url = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "/";

    _classCallCheck(this, Api);

    if (!url.endsWith("/")) {
      url = url + "/";
    }

    this.appId = projectId;
    this.url = url;
    this.options = {
      credentials: "include",
      headers: {
        "Content-Type": "application/json"
      }
    };
    this.webSocketOptions = {
      projectId: projectId
    };
    this.serviceOptions = {
      project: projectId
    };
    var client = new Client(url, this.webSocketOptions);
    this.client = client;
    this.realTime = new Realtime(projectId, client);
  }
  /**
   * Initialse the Client Api with the JWT token
   * @param {string} token - The signed JWT token received from the server on successful authentication.
   */


  _createClass(Api, [{
    key: "setToken",
    value: function setToken(token) {
      this.serviceOptions.token = token;
      this.webSocketOptions.token = token;
      this.options.headers.Authorization = "Bearer " + token;
    }
    /**
     * Set the new Project Id
     * @param {string} projectId - The Project Id.
     */

  }, {
    key: "setProjectId",
    value: function setProjectId(projectId) {
      this.appId = projectId;
      this.serviceOptions.project = projectId;
    }
    /**
     * Returns a Mongo DB client instance
     * @returns {external:DB} DB client instance
     * @deprecated since version v0.15.0. Will be deleted in version 1.0.0. Use DB instead.
     */

  }, {
    key: "Mongo",
    value: function Mongo() {
      return new _DB(this.appId, this.url, this.options, "mongo", this.realTime);
    }
    /**
     * Returns a Postgres DB client instance
     * @returns {external:DB} DB client instance
     * @deprecated since version v0.15.0. Will be deleted in version 1.0.0. Use DB instead.
     */

  }, {
    key: "Postgres",
    value: function Postgres() {
      return new _DB(this.appId, this.url, this.options, "sql-postgres", this.realTime);
    }
    /**
     * Returns a MySQL Db client instance
     * @returns {external:DB} Db client instance
     * @deprecated since version v0.15.0. Will be deleted in version 1.0.0. Use DB instead.
     */

  }, {
    key: "MySQL",
    value: function MySQL() {
      return new _DB(this.appId, this.url, this.options, "sql-mysql", this.realTime);
    }
    /**
     * Returns a DB client instance
     * @param {string} db - The alias name of the database
     * @returns {external:DB} DB client instance
     */

  }, {
    key: "DB",
    value: function DB(db) {
      return new _DB(this.appId, this.url, this.options, db, this.realTime);
    }
    /**
     * Calls an endpoint from the remote service
     * @param {string} service - The name of service
     * @param {string} endpoint - The name of endpoint to be called
     * @param {Object} params - The params to be sent to the remote service
     * @param {string} [timeout = 5000] - Timeout in milliseconds
     * @returns {Promise} Returns a promise
     * @example
     * api.call('my_service', 'my_endpoint', { msg: 'Remote services are awesome!' }, 1000)
     * .then(res => {
     *   if (res.status === 200) {
     *     // res.data contains the response given by the function
     *     console.log('Response:', res.data);
     *     return;
     *   }
     * }).catch(ex => {
     *   // Exception occured while processing request
     * });
     */

  }, {
    key: "call",
    value: function call(service, endpoint, params, timeout) {
      if (timeout === undefined) timeout = 5000;
      var url = "".concat(this.url, "v1/api/").concat(this.appId, "/services/").concat(service, "/").concat(endpoint);
      var options = Object.assign({}, this.options, {
        method: "POST",
        body: JSON.stringify({
          timeout: timeout,
          params: params
        })
      });
      return fetchAsync(url, options);
    }
    /**
     * Queues an event
     * @param {string} type - The type of event
     * @param {string} payload - Event payload
     * @returns {Promise} Returns a promise
     * @example
     * const res = await api.queueEvent("event-type", {"foo": "bar"})
     *  .delay(0)
     *  .date(new Date("2025-06-25"))
     *  .options({})
     *  .apply()
     */

  }, {
    key: "queueEvent",
    value: function queueEvent(type, payload) {
      return new QueueEvent(this.appId, this.url, this.options, type, payload);
    }
    /**
     * Returns a FileStore client instance
     * @returns {external:FileStore} FileStore client instance
     */

  }, {
    key: "FileStore",
    value: function FileStore() {
      return new _FileStore(this.appId, this.url, this.options);
    }
  }]);

  return Api;
}();
/**
 * Class representing the Queue Event Interface.
 * @example 
 * import { API, cond, or, and } from 'space-api';
 * 
 * const api = new API('my-project', 'http://localhost:4122');
 * const res = await api.queueEvent("event-type", {"foo": "bar"})
 *  .delay(0)
 *  .date(new Date("2025-06-25"))
 *  .options({})
 *  .apply()
 */


var QueueEvent =
/*#__PURE__*/
function () {
  /**
   * Create an instance of the DB Get Interface.
   * @param {string} appId
   * @param {string} url
   * @param {Object} options
   * @param {string} eventType 
   * @param {Object} eventPayload
   */
  function QueueEvent(appId, url, options, eventType, eventPayload) {
    _classCallCheck(this, QueueEvent);

    this.appId = appId;
    this.url = url;
    this.options = Object.assign({}, options, {
      method: 'POST'
    });
    this.params = {
      type: eventType,
      delay: 0,
      payload: eventPayload,
      options: {},
      isSynchronous: false
    };
  }
  /**
   * Queues the event synchronously
   */


  _createClass(QueueEvent, [{
    key: "synchronous",
    value: function synchronous() {
      this.params.isSynchronous = true;
      return this;
    }
    /**
     * Seconds to delay the webhook trigger by
     * @param {Number} delay - delay in seconds.
     */

  }, {
    key: "delay",
    value: function delay(_delay) {
      this.params.delay = _delay;
      return this;
    }
    /**
     * Date to schedule the event for
     * @param {Object} date - Date object.
     */

  }, {
    key: "date",
    value: function date(_date) {
      this.params.timestamp = _date;
      return this;
    }
    /**
     * Makes the query to queue the event. In case of synchronous events it will resolve to the response object from the webhook.
     * @returns {Promise} Returns a promise containing response from server.
     */

  }, {
    key: "apply",
    value: function apply() {
      if (!this.params.timestamp) {
        this.params.timestamp = new Date();
      }

      this.options.body = JSON.stringify(this.params);
      var url = "".concat(this.url, "v1/api/").concat(this.appId, "/eventing/queue");
      return fetchAsync(url, this.options);
    }
  }]);

  return QueueEvent;
}();

module.exports = Api;